package esb.chapter9.restaurant.test;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import javax.jms.Destination;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import junit.framework.TestCase;

import org.apache.activemq.ActiveMQConnection;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import esb.chapter9.restaurant.routing.camel.NSContext;
import esb.chapter9.restaurant.service.message.RestaurantResponse;

public class RestaurantTest extends TestCase {
	
	private List<Long> responseIDArray = new ArrayList<Long>();
	
	public void testReservation() throws Exception {
		ActiveMQConnection connection = ActiveMQConnection.makeConnection("tcp://localhost:61616");
	    connection.start();
	    Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
	    Destination destination = session.createTopic("inquiries.in");
		TextMessage tMessage = session.createTextMessage();
		tMessage.setText(createInquiry());
		Destination responseDestination = session.createQueue("lcdtv129.reponse");
		tMessage.setJMSReplyTo(responseDestination);
		MessageProducer producer = session.createProducer(destination);
		System.out.println("sending restaurant inquiry message");
		producer.send(tMessage);
		MessageConsumer consumer = session.createConsumer(responseDestination);
		List<RestaurantResponse> restaurantList = handleRestaurantResponses(consumer);
		int userChoice = getRestaurantName(restaurantList);
		String restaurantName = restaurantList.get(userChoice).getRestaurant();
		System.out.println("you chose restaurant " + restaurantName);
		Destination confDestination = session.createQueue("reservation.confirmation");
		TextMessage confMessage = session.createTextMessage();
		confMessage.setJMSReplyTo(responseDestination);
		confMessage.setText("<restaurant reservationID=\"" + responseIDArray.get(userChoice) + 
				"\">" + restaurantName + "</restaurant>");
		producer = session.createProducer(confDestination);
		producer.send(confMessage);
		TextMessage responseMessage = (TextMessage) consumer.receive();
		System.out.println("received object " + responseMessage.getText());
	}
	
	private String createInquiry() throws Exception {
		System.out.println("Welcome to our restaurant table reservation application");
		System.out.println("Please provide your guest name?");
		BufferedReader infile = new BufferedReader(new InputStreamReader(System.in));
		String guestName = infile.readLine();
		System.out.println("Please provide your room number?");
		infile = new BufferedReader(new InputStreamReader(System.in));
		String roomNumber = infile.readLine();
		System.out.println("How many persons can we include in your reservation?");
		infile = new BufferedReader(new InputStreamReader(System.in));
		String personsValue = infile.readLine();
		System.out.println("And which time can we set for your reservation? (hh:mm)");
		infile = new BufferedReader(new InputStreamReader(System.in));
		String timeValue = infile.readLine();
		Calendar reservationCal = new GregorianCalendar();
		reservationCal.set(Calendar.HOUR_OF_DAY, Integer.valueOf(timeValue.substring(0,2)));
		reservationCal.set(Calendar.MINUTE, Integer.valueOf(timeValue.substring(3)));
		DateFormat simpleFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		String strReservationTime = simpleFormat.format(reservationCal.getTime());
		String inquiry = "<processInquiry><RestaurantInquiry xmlns=\"http://message.service.restaurant.chapter9.esb\">" +
					"<guestName>" + guestName + "</guestName>" +
			  		"<persons>" + personsValue + "</persons>" +
					"<reservationTime>" + strReservationTime + "</reservationTime>" +
					"<roomNumber>" + roomNumber + "</roomNumber>" +
					"</RestaurantInquiry></processInquiry>";
		System.out.println("inquiry " + inquiry);
		return inquiry;
	}
	
	private List<RestaurantResponse> handleRestaurantResponses(MessageConsumer consumer) throws Exception {
		TextMessage responseMessage = (TextMessage) consumer.receive();
		List<RestaurantResponse> restaurantList = new ArrayList<RestaurantResponse>();
		System.out.println("receiving restaurant response message 1 " + responseMessage.getText());
		RestaurantResponse restaurantResponse1 = createResponse(responseMessage.getText());
		if(restaurantResponse1.isTableAvailable()) {
			responseIDArray.add(getReservationID(responseMessage.getText()));
			restaurantList.add(restaurantResponse1);
		}
		responseMessage = (TextMessage) consumer.receive();
		System.out.println("receiving restaurant response message 2 " + responseMessage.getText());
		RestaurantResponse restaurantResponse2 = createResponse(responseMessage.getText());
		if(restaurantResponse2.isTableAvailable()) {
			responseIDArray.add(getReservationID(responseMessage.getText()));
			restaurantList.add(restaurantResponse2);
		}
		responseMessage = (TextMessage) consumer.receive();
		System.out.println("receiving restaurant response message 3 " + responseMessage.getText());
		RestaurantResponse restaurantResponse3 = createResponse(responseMessage.getText());
		if(restaurantResponse3.isTableAvailable()) {
			responseIDArray.add(getReservationID(responseMessage.getText()));
			restaurantList.add(restaurantResponse3);
		}
		return restaurantList;
	}
	
	private RestaurantResponse createResponse(String payload) throws Exception {
		XPath xpath = XPathFactory.newInstance().newXPath();
        NSContext ctx = new NSContext();
        ctx.setNamespace("esb", "http://esbinaction.com/restaurant");
        xpath.setNamespaceContext(ctx);
        XPathExpression pathExpr = xpath.compile("//esb:out");
        Node outNode = (Node) pathExpr.evaluate(new InputSource(
                new StringReader(payload)), XPathConstants.NODE);
        NodeList restaurantNodeList = outNode.getChildNodes();
        String response = null;
        String restaurant = null;
        boolean tableAvailable = false;
        for(int i = 0; i < restaurantNodeList.getLength(); i++) {
        	Node restaurantNode = restaurantNodeList.item(i);
        	if("response".equals(restaurantNode.getLocalName())) {
        		response = restaurantNode.getTextContent();
        	} else if("restaurant".equals(restaurantNode.getLocalName())) {
        		restaurant = restaurantNode.getTextContent();
        	} else if("tableAvailable".equals(restaurantNode.getLocalName())) {
        		tableAvailable = Boolean.parseBoolean(restaurantNode.getTextContent());
        	}
        }
        RestaurantResponse responseBean = new RestaurantResponse();
        responseBean.setResponse(response);
        responseBean.setRestaurant(restaurant);
        responseBean.setTableAvailable(tableAvailable);
        return responseBean;
	}
	
	private Long getReservationID(String payload) throws Exception {
		XPath xpath = XPathFactory.newInstance().newXPath();
        NSContext ctx = new NSContext();
        ctx.setNamespace("esb", "http://esbinaction.com/restaurant");
        xpath.setNamespaceContext(ctx);
        XPathExpression pathExpr = xpath.compile("//esb:out");
        Node outNode = (Node) pathExpr.evaluate(new InputSource(
                new StringReader(payload)), XPathConstants.NODE);
        Node reservationIDNode = outNode.getAttributes().getNamedItem("reservationID");
        return Long.valueOf(reservationIDNode.getNodeValue());
	}
	
	private int getRestaurantName(List<RestaurantResponse> restaurantList) throws Exception {
		boolean userresponse = true;
		while(userresponse) {
			for(int i = 0; i < restaurantList.size(); i++) {
				System.out.println(i + 1 + ". " + restaurantList.get(i).getRestaurant() + 
						" with response " + restaurantList.get(i).getResponse());
			}
			System.out.println("Please provide your choice to confirm");
			BufferedReader infile = new BufferedReader(new InputStreamReader(System.in));
			String line = infile.readLine();
			if(line.length() != 1 || !Character.isDigit(line.charAt(0))) {
				System.out.println("Please provide a value of 1 to " + restaurantList.size());
			} else {
				int intResponse = Integer.valueOf(line);
				if(intResponse >= 1 && intResponse <= restaurantList.size()) {
					return intResponse - 1;
				} else {
					System.out.println("Please provide a value of 1 to " + restaurantList.size());
				}
			}
		}
		// we should never come here
		return -1;
	}
	
}
