package esb.chapter11.scubadiving.servicemix;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;

import org.apache.log4j.Logger;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.ComponentSupport;

import esb.chapter11.scubadiving.bean.TaxiIF;
import esb.chapter11.scubadiving.model.TaxiBooking;
import esb.chapter11.scubadiving.model.TaxiRequest;
import esb.util.framework.JiBXUtil;

public class TaxiService extends ComponentSupport implements MessageExchangeListener {
	
	private final static Logger logger = Logger.getLogger(TaxiService.class);
	@Resource
	private DeliveryChannel channel;
	private TaxiIF taxi;
	
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
    	if (exchange.getStatus() != ExchangeStatus.ACTIVE) {
    		logger.info("state is not active so ignoring call");
    		return;
    	}
    	TaxiRequest taxiRequest = null;
    	try {
    		Source contentSource = getInMessage(exchange).getContent();
    		taxiRequest = (TaxiRequest) JiBXUtil.unmarshalDocument(
    				contentSource, TaxiRequest.class);
        } catch(Exception e) {
        	logger.error("error while reading payload", e);
        }
        TaxiBooking taxiBooking = taxi.processTaxiRequest(taxiRequest);
        NormalizedMessage outMessage = exchange.createMessage();
    	try {
    		outMessage.setContent(JiBXUtil.marshalDocument(taxiBooking, "UTF-8"));
    	} catch(Exception e) {
    		logger.error("error setting taxi booking message", e);
    	}
        exchange.setMessage(outMessage, "out");
        channel.send(exchange);
    }

	public void setTaxi(TaxiIF taxi) {
		this.taxi = taxi;
	}
}
