package esb.chapter8.security;

import java.security.Principal;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.security.auth.Subject;
import javax.xml.transform.TransformerException;

import org.apache.log4j.Logger;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.security.GroupPrincipal;
import org.apache.servicemix.jbi.security.UserPrincipal;

public class SimpleAuthenticationComponent extends ComponentSupport implements MessageExchangeListener {

	private final static Logger logger = Logger.getLogger(SimpleAuthenticationComponent.class);
	private SourceTransformer sourceTransformer = new SourceTransformer();
	@Resource
	private DeliveryChannel channel;

    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        // lets dump the incoming message
        NormalizedMessage message = getInMessage(exchange);
        try {
        	logger.info("received payload " + sourceTransformer.toString(message.getContent()));
        } catch(TransformerException e) {
        	logger.error("error while reading payload", e);
        }
        Subject subject = message.getSecuritySubject();
        for (Principal principal : subject.getPrincipals()) {
        	if(principal instanceof UserPrincipal) {
        		UserPrincipal userPrincipal = (UserPrincipal) principal;
        		logger.info("request made with username " + userPrincipal.getName());
        	} else if(principal instanceof GroupPrincipal) {
        		GroupPrincipal groupPrincipal = (GroupPrincipal) principal;
        		logger.info("authenticated user belongs to group " + groupPrincipal.getName());
        	}
        }
        exchange.setStatus(ExchangeStatus.DONE);
        channel.send(exchange);
    }
}
